package lcsb.mapviewer.api.projects.models.bioEntities.elements;

import java.awt.geom.Rectangle2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;

import lcsb.mapviewer.api.BaseRestImpl;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.model.map.species.Species;
import lcsb.mapviewer.services.SecurityException;
import lcsb.mapviewer.services.UserAccessException;
import lcsb.mapviewer.services.interfaces.IModelService;
import lcsb.mapviewer.services.view.AnnotationViewFactory;
import lcsb.mapviewer.services.view.OverviewImageViewFactory;

@Transactional(value = "txManager")
public class ElementsRestImpl extends BaseRestImpl {

	Logger													 logger	= Logger.getLogger(ElementsRestImpl.class);

	@Autowired
	private IModelService						 modelService;

	@Autowired
	AnnotationViewFactory						 annotationViewFactory;

	@Autowired
	private OverviewImageViewFactory factory;

	public List<Map<String, Object>> getElements(String projectId, String id, String columns, String modelId, String token)
			throws UserAccessException, SecurityException {
		Set<Integer> ids = new HashSet<>();
		if (!id.equals("")) {
			for (String str : id.split(",")) {
				ids.add(Integer.valueOf(str));
			}
		}
		Set<String> columnsSet = createElementColumnSet(columns);

		List<Map<String, Object>> result = new ArrayList<>();

		List<Model> models = getModels(projectId, modelId, token);

		for (Model model2 : models) {
			for (Element element : model2.getElements()) {
				if (ids.size() == 0 || ids.contains(element.getId())) {
					result.add(preparedElement(element, columnsSet));
				}
			}
		}

		return result;
	}

	private Map<String, Object> preparedElement(Element element, Set<String> columnsSet) {
		Map<String, Object> result = new HashMap<>();
		for (String string : columnsSet) {
			String column = string.toLowerCase();
			Object value = null;
			if (column.equals("id") || column.equals("idobject")) {
				value = element.getId();
			} else if (column.equals("modelid")) {
				value = element.getModelData().getId();
			} else if (column.equals("name")) {
				value = element.getName();
			} else if (column.equals("type")) {
				value = element.getStringType();
			} else if (column.equals("symbol")) {
				value = element.getSymbol();
			} else if (column.equals("fullname")) {
				value = element.getFullName();
			} else if (column.equals("abbreviation")) {
				value = element.getAbbreviation();
			} else if (column.equals("compartmentid")) {
				if (element.getCompartment() != null) {
					value = element.getCompartment().getId();
				}
			} else if (column.equals("complexid")) {
				if (element instanceof Species) {
					if (((Species) element).getComplex() != null) {
						value = ((Species) element).getComplex().getId();
					}
				}
			} else if (column.equals("references")) {
				value = annotationViewFactory.createList(element.getMiriamData());
			} else if (column.equals("synonyms")) {
				value = element.getSynonyms();
			} else if (column.equals("formula")) {
				value = element.getFormula();
			} else if (column.equals("description")) {
				value = element.getNotes();
			} else if (column.equals("formersymbols")) {
				value = element.getFormerSymbols();
			} else if (column.equals("hierarchyvisibilitylevel")) {
				value = element.getVisibilityLevel();
			} else if (column.equals("bounds")) {
				value = new Rectangle2D.Double(element.getX(), element.getY(), element.getWidth(), element.getHeight());
			} else {
				value = "Unknown column";
			}
			result.put(string, value);
		}
		return result;
	}

	private Set<String> createElementColumnSet(String columns) {
		Set<String> columnsSet = new HashSet<>();
		if (columns.equals("")) {
			columnsSet.add("id");
			columnsSet.add("modelId");
			columnsSet.add("name");
			columnsSet.add("type");
			columnsSet.add("description");
			columnsSet.add("type");
			columnsSet.add("symbol");
			columnsSet.add("complexId");
			columnsSet.add("compartmentId");
			columnsSet.add("fullName");
			columnsSet.add("abbreviation");
			columnsSet.add("formula");
			columnsSet.add("name");
			columnsSet.add("synonyms");
			columnsSet.add("formerSymbols");
			columnsSet.add("references");
			columnsSet.add("bounds");
			columnsSet.add("hierarchyVisibilityLevel");
		} else {
			for (String str : columns.split(",")) {
				columnsSet.add(str);
			}
		}
		return columnsSet;
	}

	/**
	 * @return the modelService
	 * @see #modelService
	 */
	public IModelService getModelService() {
		return modelService;
	}

	/**
	 * @param modelService
	 *          the modelService to set
	 * @see #modelService
	 */
	public void setModelService(IModelService modelService) {
		this.modelService = modelService;
	}

	/**
	 * @return the factory
	 * @see #factory
	 */
	public OverviewImageViewFactory getFactory() {
		return factory;
	}

	/**
	 * @param factory
	 *          the factory to set
	 * @see #factory
	 */
	public void setFactory(OverviewImageViewFactory factory) {
		this.factory = factory;
	}
}
