package lcsb.mapviewer.api.projects.overlays;

import java.io.IOException;
import java.util.List;
import java.util.Map;

import javax.ws.rs.PathParam;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import lcsb.mapviewer.api.BaseController;
import lcsb.mapviewer.api.QueryException;
import lcsb.mapviewer.model.cache.FileEntry;
import lcsb.mapviewer.services.SecurityException;
import lcsb.mapviewer.services.view.LayoutView;

@RestController
public class OverlayController extends BaseController {

	@Autowired
	private OverlayRestImpl overlayController;

	@RequestMapping(value = "/projects/{projectId}/overlays/", method = { RequestMethod.GET }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public List<LayoutView> getOverlayList(//
			@RequestParam(value = "token") String token, //
			@PathVariable(value = "projectId") String projectId //
	) throws SecurityException, QueryException {
		return overlayController.getOverlayList(token, projectId);
	}

	@RequestMapping(value = "/projects/{projectId}/overlays/{overlayId}/", method = { RequestMethod.GET }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public LayoutView getOverlayById(//
			@RequestParam(value = "token") String token, //
			@PathVariable(value = "projectId") String projectId, //
			@PathVariable(value = "overlayId") String overlayId//
	) throws SecurityException, QueryException {
		return overlayController.getOverlayById(token, projectId, overlayId);
	}

	@RequestMapping(value = "/projects/{projectId}/overlays/{overlayId}/bioEntities/", method = { RequestMethod.GET }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public List<Map<String, Object>> getOverlayElements(
			@RequestParam(value = "token") String token, 
			@PathVariable(value = "projectId") String projectId, //
			@PathVariable(value = "overlayId") String overlayId, 
			@RequestParam(value = "columns", defaultValue = "") String columns
			)
			throws SecurityException, QueryException {
		return overlayController.getOverlayElements(token, projectId, Integer.valueOf(overlayId), columns);
	}

	@RequestMapping(value = "/overlay/getOverlayTypes", method = { RequestMethod.GET }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public List<Map<String, Object>> getOverlayTypes(@RequestParam(value = "token") String token) throws SecurityException, QueryException {
		return overlayController.getOverlayTypes(token);
	}

	@RequestMapping(value = "/overlay/updateOverlay", method = { RequestMethod.GET, RequestMethod.PUT, RequestMethod.POST },
			produces = { MediaType.APPLICATION_JSON_VALUE })
	public Map<String, Object> updateOverlay(@RequestParam(value = "token") String token, @RequestParam(value = "projectId") String projectId,
			@RequestParam(value = "overlayId") String overlayId, @RequestParam(value = "name") String name, @RequestParam(value = "description") String description)
			throws SecurityException, QueryException {
		return overlayController.updateOverlay(token, projectId, overlayId, name, description);
	}

	@RequestMapping(value = "/overlay/addOverlay", method = { RequestMethod.GET, RequestMethod.POST }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public Map<String, Object> addOverlay(@RequestParam(value = "token") String token, @RequestParam(value = "projectId") String projectId,
			@RequestParam(value = "name") String name, @RequestParam(value = "description") String description, @RequestParam(value = "content") String content,
			@RequestParam(value = "filename") String filename, @RequestParam(value = "type", defaultValue = "") String type)
			throws SecurityException, QueryException, IOException {
		return overlayController.addOverlay(token, projectId, name, description, content, filename, type);
	}

	@RequestMapping(value = "/overlay/removeOverlay", method = { RequestMethod.GET, RequestMethod.DELETE, RequestMethod.POST },
			produces = { MediaType.APPLICATION_JSON_VALUE })
	public Map<String, Object> removeOverlay(@RequestParam(value = "token") String token, @RequestParam(value = "projectId") String projectId,
			@RequestParam(value = "overlayId") String overlayId) throws SecurityException, QueryException, IOException {
		return overlayController.removeOverlay(token, projectId, overlayId);
	}

	@RequestMapping(value = "/overlay/getOverlaySource", method = { RequestMethod.GET }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public ResponseEntity<byte[]> getOverlaySource(@RequestParam(value = "token") String token, @RequestParam(value = "projectId") String projectId,
			@RequestParam(value = "overlayId") String overlayId) throws SecurityException, QueryException {

		FileEntry file = overlayController.getOverlaySource(token, projectId, overlayId);
		MediaType type = MediaType.TEXT_PLAIN;
		if (file.getOriginalFileName().endsWith("xml")) {
			type = MediaType.APPLICATION_XML;
		}
		return ResponseEntity
				.ok().contentLength(file.getFileContent().length).contentType(type).header("Content-Disposition", "attachment; filename=" + file.getOriginalFileName())
				.body(file.getFileContent());
	}

	@RequestMapping(value = "/overlay/getOverlayElement", method = { RequestMethod.GET, RequestMethod.POST }, produces = { MediaType.APPLICATION_JSON_VALUE })
	public Map<String, Object> getFullElement(@RequestParam(value = "token") String token, @RequestParam(value = "projectId") String projectId,
			@RequestParam(value = "modelId") String modelId, @RequestParam(value = "overlayId") String overlayId,
			@RequestParam(value = "elementId") String elementId, @RequestParam(value = "elementType") String elementType,
			@RequestParam(value = "columns", defaultValue = "") String columns) throws SecurityException, QueryException {
		return overlayController
				.getOverlayElement(token, projectId, Integer.valueOf(modelId), Integer.valueOf(overlayId), Integer.valueOf(elementId), elementType, columns);
	}

	/**
	 * @return the overlayController
	 * @see #overlayController
	 */
	public OverlayRestImpl getOverlayController() {
		return overlayController;
	}

	/**
	 * @param overlayController
	 *          the overlayController to set
	 * @see #overlayController
	 */
	public void setOverlayController(OverlayRestImpl overlayController) {
		this.overlayController = overlayController;
	}

}